/*
Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package soleil;

import java.awt.Adjustable;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.SwingConstants;

public class MouvementSoleil extends JApplet implements AdjustmentListener {
    
    static final double minDelta = -23.5 * (Math.PI/180);
    static final double maxDelta = 23.5 * (Math.PI/180);
    static final double minPhi = -90 * (Math.PI/180);
    static final double maxPhi = 90 * (Math.PI/180);
    static final double H = Math.PI / 3; // angle horaire
    static final double deltaInitial = 15; // en degrs
    static final Color couleurEquateur = new Color(150, 255, 200, 200);
    static final Color couleurTexteEquateur = new Color(100, 200, 150);
    static final Color couleurHorizon = new Color(250, 210, 170, 200);
    static final Color couleurTexteHorizon = new Color(200, 150, 100);
    static final Color couleurSoleil = new Color(255, 255, 0);
    static final Color couleurSoleilCouche = new Color(30, 30, 30);
    static final Color couleurFlecheSoleil = new Color(255, 150, 0);
    static final Color couleurPositionsSoleil = new Color(255, 100, 0);
    static final Color couleurDelta = new Color(255, 50, 50);
    static final Color couleurPhi = new Color(50, 50, 255);
    static final Color couleurArcs = Color.lightGray;
    static final Color couleurAngleA = new Color(150, 100, 50);
    static final Color couleurAngleH = new Color(50, 150, 100);
    static final Color couleurAngleHauteur = new Color(150, 50, 150);
    static final int precisionCourbes = 100;
    int imgdimx;
    int imgdimy;
    protected ImageCanvas ic;
    protected Image croquis = null;
    protected Graphics2D g2d = null;
    protected JScrollBar sliderDelta, sliderPhi;
    protected double delta;
    protected double phi;
    protected int cx, cy; // position du centre de l'image en pixels
    protected int rayon; // rayon du grand cercle en pixels
    protected JLabel labelDelta, labelPhi;
    protected Font police;
    protected Matrice rotObservateur; // matrice de rotation par rapport  l'observateur
    protected Matrice rotEquateur; // matrice de rotation de l'quateur cleste par rapport  l'horizon
    protected Matrice rotObsEq; // rotObservateur * rotEquateur
    protected Vecteur O, Nord, Sud, Est, Ouest, Z; // points en coordonnes sphriques
    protected String titreDelta, titrePhi;
    
    
    public void init() {
        imgdimx = Math.min(getWidth(), getHeight() - 50);
        if (imgdimx <= 0)
            imgdimx = 500;
        imgdimy = imgdimx;
        getContentPane().setLayout(new BorderLayout());
        getContentPane().setBackground(Color.white);
        
        creerCroquis();
        ic = new ImageCanvas(imgdimx, imgdimy);
        getContentPane().add(ic, BorderLayout.CENTER);
        
        JPanel basPane = new JPanel(new GridLayout(2, 2, 5, 5));
        basPane.setBackground(Color.white);
        getContentPane().add(basPane, BorderLayout.SOUTH);
        
        delta = deltaInitial * (Math.PI/180);
        if (police.canDisplay('\u03B4'))
            titreDelta = "\u03B4";
        else
            titreDelta = "delta";
        labelDelta = new JLabel(titreDelta + " = " + Math.round(10 * delta*180/Math.PI)/10.0 + "", SwingConstants.RIGHT);
        labelDelta.setForeground(couleurDelta);
        labelDelta.setFont(police);
        basPane.add(labelDelta);
        sliderDelta = new JScrollBar(Adjustable.HORIZONTAL, (int)Math.round(100 * (delta - minDelta) / (maxDelta - minDelta)), 10, 0, 100+10);
        sliderDelta.setBlockIncrement(5);
        sliderDelta.addAdjustmentListener(this);
        basPane.add(sliderDelta);
        
        phi = 50 * (Math.PI/180);
        if (police.canDisplay('\u03C6'))
            titrePhi = "\u03C6";
        else
            titrePhi = "phi";
        labelPhi = new JLabel(titrePhi + " = " + Math.round(10 * phi*180/Math.PI)/10.0 + "", SwingConstants.RIGHT);
        labelPhi.setForeground(couleurPhi);
        labelPhi.setFont(police);
        basPane.add(labelPhi);
        sliderPhi = new JScrollBar(Adjustable.HORIZONTAL, (int)Math.round(100 * (phi - minPhi) / (maxPhi - minPhi)), 10, 0, 100+10);
        sliderPhi.setBlockIncrement(5);
        sliderPhi.addAdjustmentListener(this);
        basPane.add(sliderPhi);
        
        rotObservateur = new Matrice(3, 3);
        rotEquateur = new Matrice(3, 3);
        O = new Vecteur(0, 0, 0);
        Nord = new Vecteur(1, Math.PI / 2, -Math.PI / 2);
        Sud = new Vecteur(1, Math.PI / 2, Math.PI / 2);
        Est = new Vecteur(1, Math.PI / 2, Math.PI);
        Ouest = new Vecteur(1, Math.PI / 2, 0);
        Z = new Vecteur(1, 0, 0);
        
        majCroquis();
        
        validate();
        if (ic != null)
            ic.requestFocus();
    }
    
    public void adjustmentValueChanged(AdjustmentEvent e) {
        Adjustable adj = e.getAdjustable();
        if (adj == sliderDelta) {
            double ndelta = minDelta + (maxDelta - minDelta) * (e.getValue() / 100.0);
            if (ndelta != delta) {
                delta = ndelta;
                labelDelta.setText(titreDelta + " = " + Math.round(10 * delta*180/Math.PI)/10.0 + "");
                majCroquis();
            }
        } else if (adj == sliderPhi) {
            double nphi = minPhi + (maxPhi - minPhi) * (e.getValue() / 100.0);
            if (nphi != phi) {
                phi = nphi;
                labelPhi.setText(titrePhi + " = " + Math.round(10 * phi*180/Math.PI)/10.0 + "");
                majCroquis();
            }
        }
    }
    
    protected void creerCroquis() {
        croquis = createImage(imgdimx, imgdimy);
        g2d = (Graphics2D)croquis.getGraphics();
        police = new Font(getFont().getName(), Font.BOLD, 12);
        g2d.setFont(police);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
    }
    
    protected void majCroquis() {
        final double alpha = 0.2; // angle d'observation
        rotObservateur.set(0, 0, Math.cos(alpha));
        rotObservateur.set(0, 1, 0);
        rotObservateur.set(0, 2, -Math.sin(alpha));
        rotObservateur.set(1, 0, 0);
        rotObservateur.set(1, 1, 1);
        rotObservateur.set(1, 2, 0);
        rotObservateur.set(2, 0, Math.sin(alpha));
        rotObservateur.set(2, 1, 0);
        rotObservateur.set(2, 2, Math.cos(alpha));
        
        final double cphi = Math.PI / 2 - phi; // angle de l'quateur par rapport  l'horizon
        rotEquateur.set(0, 0, 1);
        rotEquateur.set(0, 1, 0);
        rotEquateur.set(0, 2, 0);
        rotEquateur.set(1, 0, 0);
        rotEquateur.set(1, 1, Math.cos(cphi));
        rotEquateur.set(1, 2, Math.sin(cphi));
        rotEquateur.set(2, 0, 0);
        rotEquateur.set(2, 1, -Math.sin(cphi));
        rotEquateur.set(2, 2, Math.cos(cphi));
        
        rotObsEq = rotObservateur.produit(rotEquateur);
        
        g2d.setColor(Color.white);
        g2d.fillRect(0, 0, imgdimx, imgdimy);
        //g2d.setColor(Color.black);
        //g2d.drawRect(0, 0, imgdimx-1, imgdimy-1);
        cx = imgdimx / 2;
        cy = imgdimy / 2;
        rayon = Math.min(cx - 20, cy - 20); // rayon de la sphre en pixels
        
        /*
        fond bleu
        for (int i=0; i<100; i++) {
            g2d.setColor(new Color((int)(205 + i / 2), (int)(205 + i / 2), 255));
            g2d.fill(new Ellipse2D.Double(cx - (1 - i / 100.0) * rayon, cy - (1 - i / 100.0) * rayon, (2 - i / 50.0) * rayon, (2 - i / 50.0) * rayon));
        }
        */
        g2d.setColor(new Color(230, 230, 255));
        g2d.fillOval(cx - rayon, cy - rayon, rayon * 2, rayon * 2);
        
        g2d.setColor(Color.darkGray);
        g2d.drawOval(cx - rayon, cy - rayon, rayon * 2, rayon * 2);
        
        final Vecteur S = new Vecteur(1, Math.PI / 2 - delta, Math.PI / 2 - H); // position du soleil dans le repre de l'quateur
        final Vecteur Scart = S.spheriquesVersCartesiennes();
        final Vecteur ScartHoz = rotEquateur.produit(Scart); // rotation quateur -> horizon
        final Vecteur SsphHoz = ScartHoz.cartesiennesVersSpheriques(); // coordonnes sphriques de S dans le repre de l'horizon
        final Point2D ptS = conversion(S, true);
        final Point2D ptO = new Point2D.Double(cx, cy);
        final Vecteur M = new Vecteur(1, Math.PI / 2, Math.PI / 2 - H); // repre quatorial
        final Point2D ptM = conversion(M, true);
        final Point2D ptNord = conversion(Nord, false);
        final Vecteur P = new Vecteur(1, Math.PI / 2 - phi, -Math.PI / 2);
        final Point2D ptP = conversion(P, false);
        final Vecteur N = new Vecteur(1, Math.PI / 2, SsphHoz.get(2)); // repre de l'horizon
        Point2D pt;
        
        // positions du soleil (derrire)
        dessinPositionsSoleil(false);
        
        /*
        // angle delta : OS
        g2d.setColor(couleurDelta);
        if (((phi > 0 && delta < 0) || (phi <= 0 && delta > 0)) && SsphHoz.get(1) > Math.PI / 2)
            g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptS.getX(), ptS.getY()));
        
        // angle phi : OP
        if (phi < 0) {
            g2d.setColor(couleurPhi);
            g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptP.getX(), ptP.getY()));
        }
        */
        
        // calcul cercle de l'quateur cleste (derrire)
        final GeneralPath cheminEq1 = new GeneralPath();
        pt = conversion(new Vecteur(1, Math.PI / 2, 0), true);
        cheminEq1.moveTo((float)pt.getX(), (float)pt.getY());
        for (int i=1; i<=precisionCourbes; i++) {
            pt = conversion(new Vecteur(1, Math.PI / 2, - (i / (double)precisionCourbes) * Math.PI), true);
            cheminEq1.lineTo((float)pt.getX(), (float)pt.getY());
        }
        cheminEq1.closePath();
        // affichage cercle de l'quateur cleste (derrire)
        g2d.setColor(couleurEquateur);
        g2d.fill(cheminEq1);
        
        dessinHorizon();
        
        /*
        // angle delta : OS
        g2d.setColor(couleurDelta);
        if (((phi > 0 && delta < 0) || (phi <= 0 && delta > 0)) && SsphHoz.get(1) <= Math.PI / 2)
            g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptS.getX(), ptS.getY()));
        
        // angle phi : O Nord
        if (phi < 0) {
            g2d.setColor(couleurPhi);
            g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptNord.getX(), ptNord.getY()));
        }
        */
        
        // calcul cercle de l'quateur cleste (devant)
        final GeneralPath cheminEq2 = new GeneralPath();
        pt = conversion(new Vecteur(1, Math.PI / 2, 0), true);
        cheminEq2.moveTo((float)pt.getX(), (float)pt.getY());
        for (int i=1; i<=precisionCourbes/2; i++) {
            pt = conversion(new Vecteur(1, Math.PI / 2, (i / (double)(precisionCourbes/2)) * Math.PI), true);
            cheminEq2.lineTo((float)pt.getX(), (float)pt.getY());
        }
        cheminEq2.closePath();
        // affichage cercle de l'quateur cleste (devant)
        g2d.setColor(couleurEquateur);
        g2d.fill(cheminEq2);
        g2d.setColor(couleurTexteEquateur);
        if (phi > 0) {
            pt = conversion(Ouest, false);
            g2d.rotate(phi - Math.PI/2, pt.getX(), pt.getY());
            g2d.drawString("Equateur cleste", (float)(pt.getX() - rayon * 0.75), (float)pt.getY() + 10);
            g2d.rotate(Math.PI/2 - phi, pt.getX(), pt.getY());
        } else {
            pt = conversion(Est, false);
            g2d.rotate(phi + Math.PI/2, pt.getX(), pt.getY());
            g2d.drawString("Equateur cleste", (float)(pt.getX() - rayon * 0.75), (float)pt.getY() - 5);
            g2d.rotate(-phi - Math.PI/2, pt.getX(), pt.getY());
        }
        
        // positions du soleil (devant)
        dessinPositionsSoleil(true);
        
        // soleil
        dessinSoleil(S, ptS, SsphHoz);
        
        // arc ZSN
        dessinArcZSN(Z, SsphHoz, N);
        
        // arc PSM
        dessinArcPSM(P, S, M);
        
        /*
        // angle delta : OS
        g2d.setColor(couleurDelta);
        if ((phi > 0 && delta >= 0) || (phi <= 0 && delta <= 0))
            g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptS.getX(), ptS.getY()));
        // angle delta : OM
        g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptM.getX(), ptM.getY()));
        */
        
        dessinAngleDelta(M, S);
        
        dessinAnglePhi(P);
        
        // angle A (azimuth du Soleil)
        dessinAngleA(N);
        
        // angle H (angle horaire)
        dessinAngleH(M);
        
        // angle h (hauteur du soleil)
        dessinAngleHauteur(N, SsphHoz);
        
        // points
        g2d.setColor(Color.black);
        
        // affichage des points fixes
        afficherPoint(O, false, true, "O");
        afficherPoint(Nord, false, true, "Nord");
        afficherPoint(Sud, false, true, "Sud");
        afficherPoint(Est, false, true, "Est");
        afficherPoint(Ouest, false, true, "Ouest");
        afficherPoint(Z, false, true, "Z");
        
        // affichage des points en mouvement
        afficherPoint(P, false, true, "P");
        
        afficherPoint(M, true, true, "M");
        afficherPoint(N, false, true, "N");
        afficherPoint(S, true, true, "S");
        
        ic.updateImage(croquis);
    }
    
    protected void dessinHorizon() {
        final GeneralPath cheminHorizon = new GeneralPath();
        Point2D pt = conversion(new Vecteur(1, Math.PI / 2, 0), false);
        cheminHorizon.moveTo((float)pt.getX(), (float)pt.getY());
        for (int i=1; i<precisionCourbes/2; i++) {
            pt = conversion(new Vecteur(1, Math.PI / 2, (i / (double)(precisionCourbes/2)) * (2 * Math.PI)), false);
            cheminHorizon.lineTo((float)pt.getX(), (float)pt.getY());
        }
        cheminHorizon.closePath();
        g2d.setColor(couleurHorizon);
        g2d.fill(cheminHorizon);
        g2d.setColor(couleurTexteHorizon);
        pt = conversion(Ouest, false);
        g2d.rotate(-0.1, pt.getX(), pt.getY());
        g2d.drawString("Horizon cleste", (float)(pt.getX() + rayon / 4), (float)pt.getY() + 20);
        g2d.rotate(0.1, pt.getX(), pt.getY());
    }
    
    protected void dessinPositionsSoleil(final boolean devant) {
        g2d.setColor(couleurPositionsSoleil);
        Point2D pt;
        for (int i=0; i<precisionCourbes; i++) {
            pt = conversion(new Vecteur(1, Math.PI / 2 - delta, (i / (double)precisionCourbes) *  (2 * Math.PI)), true);
            if ((i <= precisionCourbes/4 || i >= (precisionCourbes*3)/4) == devant)
                g2d.fill(new Ellipse2D.Double(pt.getX() - 1, pt.getY() - 1, 2, 2));
        }
    }
    
    protected void dessinSoleil(final Vecteur S, final Point2D ptS, final Vecteur SsphHoz) {
        if (SsphHoz.get(1) <= Math.PI/2)
            g2d.setColor(couleurSoleil);
        else
            g2d.setColor(couleurSoleilCouche);
        g2d.fill(new Ellipse2D.Double(ptS.getX() - 5, ptS.getY() - 5, 10, 10));
        // flche de direction du soleil
        final Vecteur S2 = new Vecteur(1, S.get(1), S.get(2) - Math.PI/16);
        g2d.setColor(couleurFlecheSoleil);
        dessinArc(S, S2, true, true);
    }
    
    protected void dessinArcZSN(final Vecteur Z, final Vecteur SsphHoz, final Vecteur N) {
        g2d.setColor(couleurArcs);
        final Vecteur Zbis = new Vecteur(1, 0, SsphHoz.get(2)); // Z avec le mme phi que SsphHoz
        dessinArc(Zbis, SsphHoz.get(1) < N.get(1) ? SsphHoz : N, false, false);
    }
    
    protected void dessinArcPSM(final Vecteur P, final Vecteur S, final Vecteur M) {
        g2d.setColor(couleurArcs);
        final Vecteur PEq = new Vecteur(1, 0, S.get(2)); // P dans le repre quatorial, avec le mme phi que S
        dessinArc(PEq, S.get(1) < M.get(1) ? S : M, true, false);
    }
    
    protected void dessinAngleDelta(final Vecteur M, final Vecteur S) {
        g2d.setColor(couleurDelta);
        dessinArc(M, S, true, true);
        if (Math.abs(delta) > 0.1) {
            double r;
            if ((phi >= 0 && delta >= 0) || (phi < 0 && delta < 0))
                r = 1.1;
            else
                r = 0.9;
            Point2D pt = conversion(new Vecteur(r, Math.PI/2 - (Math.PI/2 - S.get(1))/2, M.get(2)), true);
            g2d.drawString(titreDelta, (float)(pt.getX() - g2d.getFontMetrics().stringWidth(titreDelta)/2), (float)(pt.getY() + 5));
        }
    }
    
    protected void dessinAnglePhi(final Vecteur P) {
        g2d.setColor(couleurPhi);
        /*
        g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptNord.getX(), ptNord.getY()));
        g2d.draw(new Line2D.Double(ptO.getX(), ptO.getY(), ptP.getX(), ptP.getY()));
        */
        dessinArc(Nord, P, false, true);
        Point2D pt = conversion(new Vecteur(1.05, Math.PI/2 - phi/2, -Math.PI/2), false);
        g2d.drawString(titrePhi, (float)(pt.getX() - g2d.getFontMetrics().stringWidth(titrePhi)/2), (float)(pt.getY() + 5));
    }
    
    protected void dessinAngleA(final Vecteur N) {
        g2d.setColor(couleurAngleA);
        dessinArc(Sud, N, false, true);
        Point2D pt = conversion(new Vecteur(0.87, Math.PI/2, Math.PI/2 - (Math.PI/2 - N.get(2))/2), false);
        g2d.drawString("A", (float)(pt.getX() - 10), (float)(pt.getY() + 5));
    }
    
    protected void dessinAngleH(final Vecteur M) {
        g2d.setColor(couleurAngleH);
        dessinArc(new Vecteur(1, Math.PI/2, Math.PI/2), M, true, true);
        Point2D pt = conversion(new Vecteur(0.87, Math.PI/2, Math.PI/2 - (Math.PI/2 - M.get(2))/2), true);
        g2d.drawString("H", (float)(pt.getX() - 10), (float)(pt.getY() + 5));
    }
    
    protected void dessinAngleHauteur(final Vecteur N, final Vecteur SsphHoz) {
        g2d.setColor(couleurAngleHauteur);
        dessinArc(N, SsphHoz, false, true);
        if (Math.abs(SsphHoz.get(1) - Math.PI/2) > 0.1) {
            double r;
            if ((phi >= 0 && delta >= 0) || (phi < 0 && delta < 0))
                r = 0.9;
            else
                r = 1.1;
            Point2D pt = conversion(new Vecteur(r, Math.PI/2 - (Math.PI/2 - SsphHoz.get(1))/2, N.get(2)), false);
            g2d.drawString("h", (float)(pt.getX() - 5), (float)(pt.getY() + 5));
        }
    }
    
    // dessin d'un arc entre 2 points dans l'un des 2 systmes de coordonnes sphriques, en suivant l'une des coordonnes
    protected void dessinArc(final Vecteur v1, final Vecteur v2, final boolean coordEq, final boolean fleche) {
        final int nb = precisionCourbes/4;
        final double dr = (v2.get(0) - v1.get(0)) / nb;
        final double dtheta = (v2.get(1) - v1.get(1)) / nb;
        final double dphi = (v2.get(2) - v1.get(2)) / nb; // ne marche pas toujours (par ex. avec theta = 0)
        final GeneralPath chemin = new GeneralPath();
        Point2D pt = conversion(v1, coordEq);
        Point2D vpt = null;
        chemin.moveTo((float)pt.getX(), (float)pt.getY());
        for (int i=1; i<=nb; i++) {
            if (i == nb)
                vpt = pt;
            pt = conversion(new Vecteur(v1.get(0) + i * dr, v1.get(1) + i * dtheta, v1.get(2) + i * dphi), coordEq);
            chemin.lineTo((float)pt.getX(), (float)pt.getY());
        }
        g2d.draw(chemin);
        if (fleche && Math.abs(v2.get(0) - v1.get(0)) + Math.abs(v2.get(1) - v1.get(1)) + Math.abs(v2.get(2) - v1.get(2)) > 0.1)
            dessinFleche(vpt, pt);
    }
    
    // dessin des 2 traits de la fin de la flche  partir des 2 derniers points d'un arc
    protected void dessinFleche(final Point2D pt1, final Point2D pt2) {
        final double l = 6; // longueur des traits en pixels
        final double da = Math.PI / 8; // angle des traits avec la fin de l'arc
        final double dx = pt2.getX() - pt1.getX();
        final double dy = pt2.getY() - pt1.getY();
        double angle;
        if (dx == 0) {
            if (dy > 0)
                angle = Math.PI / 2;
            else
                angle = -Math.PI / 2;
        } else {
            angle = Math.atan(dy / dx);
            if (dx < 0)
                angle = angle + Math.PI;
        }
        Point2D ptFleche = new Point2D.Double(pt2.getX() - l * Math.cos(angle + da), pt2.getY() - l * Math.sin(angle + da));
        g2d.draw(new Line2D.Double(pt2.getX(), pt2.getY(), ptFleche.getX(), ptFleche.getY()));
        ptFleche = new Point2D.Double(pt2.getX() - l * Math.cos(angle - da), pt2.getY() - l * Math.sin(angle - da));
        g2d.draw(new Line2D.Double(pt2.getX(), pt2.getY(), ptFleche.getX(), ptFleche.getY()));
    }
    
    // passage des coordonnes sphriques aux coordonnes cran
    protected Point2D conversion(final Vecteur vsph, final boolean rotEq) {
        // passage aux coordonnes cartsiennes
        Vecteur v = vsph.spheriquesVersCartesiennes();
        
        // rotation optionelle de l'quateur et rotation permettant de voir l'horizon de haut
        if (rotEq)
            v = rotObsEq.produit(v);
        else
            v = rotObservateur.produit(v);
        
        return(projectionEcran(v));
    }
    
    // ajout d'un point en coordonnes sphriques (r, theta, phi)
    protected void afficherPoint(final Vecteur vsph, final boolean rotEq, final boolean croix, final String titre) {
        final Point2D p = conversion(vsph, rotEq);
        
        // affichage d'une croix ou d'un point
        if (croix) {
            g2d.draw(new Line2D.Double(p.getX() - 2, p.getY(), p.getX() + 2, p.getY()));
            g2d.draw(new Line2D.Double(p.getX(), p.getY() - 2, p.getX(), p.getY() + 2));
        } else
            g2d.draw(new Line2D.Double(p.getX(), p.getY(), p.getX(), p.getY()));
        
        // affichage du titre
        if (titre != null) {
            final int l = g2d.getFontMetrics().stringWidth(titre);
            g2d.drawString(titre, (float)p.getX() - l/2, (float)p.getY() - 9);
        }
    }
    
    // projection selon l'axe x  partir des coordonnes cartsiennes, puis conversion vers l'cran
    protected Point2D projectionEcran(final Vecteur v) {
        return(new Point2D.Double(cx + v.get(1) * rayon, cy - v.get(2) * rayon));
    }
    
}
